# -*- coding: utf-8 -*-

import os
from cent.core import Client
from instant.producers import broadcast
from instant.conf import CENTRIFUGO_PORT, CENTRIFUGO_HOST, SECRET_KEY
from presence.conf import ASYNC_BACKEND, CHANNEL, GLOBAL_WORKER
if ASYNC_BACKEND == 'celery':
    from celery import task
elif ASYNC_BACKEND == 'huey':
    from huey.contrib.djhuey import crontab, periodic_task

DEBUG = False

def _update_presence():
    """
    Fetch presence info every minute from Centrifugo and post it to the clients for update.
    """
    # get presence info from Centrifugo
    cent_url = CENTRIFUGO_HOST+":"+str(CENTRIFUGO_PORT)
    client = Client(cent_url, SECRET_KEY, timeout=1)
    clients = client.presence(CHANNEL)
    if DEBUG:
        print "Updating presence"
    # post presence info
    total_users = 0
    anonymous_users = 0
    users = []
    for client in clients.keys():
        total_users += 1
        user = clients[client]["user"]
        if user == "":
            anonymous_users += 1
        if DEBUG and user != "":
            print "Client: "+str(user)
        if user not in users and user != "":
            users.append(user.encode('utf8'))
    if DEBUG:
        print str(anonymous_users)+" anonymous users"
        print "Connected users:"
        for user in users:
            print '- '+user
    # send presence info into a Centrifugo channel
    msg = ",".join(users)+'/'+str(anonymous_users)
    if total_users > 0:
        broadcast(message=msg, event_class="__presence__")
    return

def global_update():
    com = './presence/go/centpres'
    os.system(com)
    return

if ASYNC_BACKEND == 'huey':

    @periodic_task(crontab(minute='*'))
    def update_presence():
        if GLOBAL_WORKER is True:
            global_update()
        else:
            _update_presence()
        return

elif ASYNC_BACKEND == 'celery':
    
    @task(ignore_results=True)
    def update_presence():
        if GLOBAL_WORKER is True:
            global_update()
        else:
            _update_presence()
        return


# -*- coding: utf-8 -*-

from django.conf import settings


ASYNC_BACKEND =  getattr(settings, 'PRESENCE_ASYNC_BACKEND', 'celery')

SITE_SLUG =  getattr(settings, 'SITE_SLUG', 'site')
CHANNEL = getattr(settings, 'PRESENCE_CHANNEL', SITE_SLUG+'_public')

GLOBAL_WORKER = getattr(settings, 'PRESENCE_GLOBAL_WORKER', False)

FREQUENCY = getattr(settings, 'PRESENCE_FREQUENCY', 10)


from __future__ import print_function
import os
import json
from django.core.management.base import BaseCommand
from django.conf import settings
import presence
from presence.management.conf import bcolors
from instant.conf import SECRET_KEY, CENTRIFUGO_HOST, CENTRIFUGO_PORT
from presence.conf import CHANNEL, FREQUENCY


class Command(BaseCommand):
    help = 'Installs the Django Presence go worker'

    def handle(self, *args, **options):
        prespath = os.path.dirname(presence.__file__)
        gomodpath = os.path.join(prespath, 'go')
        # copy the executable
        print ("Installing the worker ...")
        _cmd = "cp -v "+gomodpath+"/centpres "+settings.BASE_DIR
        os.system(_cmd)
        print("["+bcolors.OKBLUE+"x"+bcolors.ENDC+"] Executable file installed")
        # generate config
        print("Generating config ...")
        conf = {
                "centrifugo_secret_key":SECRET_KEY,
                "centrifugo_host":CENTRIFUGO_HOST,
                "centrifugo_port":str(CENTRIFUGO_PORT),
                "channels":[CHANNEL],
                "frequency":FREQUENCY
                }
        _file = open(settings.BASE_DIR+"/centpres_config.json", "w")
        _file.write(json.dumps(conf))
        _file.close()
        print("["+bcolors.OKBLUE+"x"+bcolors.ENDC+"] File centpres_config.json generated" )
        msg = "[ "+bcolors.OKGREEN+"ok"+bcolors.ENDC+" ] Django presence worker installed: run "+bcolors.OKBLUE+"./centpres"+bcolors.ENDC+" to start the service"
        print(msg)
        return
